@@ -1,9 +1,9 @@
 /**
  * Copyright (c) 2006 IBM Corporation and others.
  * All rights reserved.   This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * are made available under the terms of the Eclipse Public License v2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * http://www.eclipse.org/legal/epl-v20.html
  * 
  * Contributors: 
  *   IBM - Initial API and implementation
@@ -16,22 +16,32 @@ import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import org.eclipse.core.resources.IFile;
+import org.eclipse.core.resources.IWorkspaceRoot;
+import org.eclipse.core.runtime.Path;
 import org.eclipse.emf.codegen.ecore.CodeGenEcorePlugin;
 import org.eclipse.emf.codegen.ecore.genmodel.GenModel;
+import org.eclipse.emf.codegen.ecore.genmodel.util.GenModelUtil;
 import org.eclipse.emf.codegen.merge.java.JControlModel;
 import org.eclipse.emf.codegen.merge.java.facade.FacadeHelper;
 import org.eclipse.emf.codegen.util.CodeGenUtil;
+import org.eclipse.emf.common.CommonPlugin;
 import org.eclipse.emf.common.notify.AdapterFactory;
 import org.eclipse.emf.common.util.BasicDiagnostic;
 import org.eclipse.emf.common.util.Diagnostic;
 import org.eclipse.emf.common.util.Monitor;
+import org.eclipse.emf.common.util.URI;
 import org.eclipse.emf.ecore.EObject;
+import org.eclipse.emf.ecore.plugin.EcorePlugin;
 import org.eclipse.emf.ecore.resource.ResourceSet;
+import org.eclipse.jdt.core.ICompilationUnit;
 import org.eclipse.jdt.core.JavaCore;
+import org.eclipse.jdt.core.formatter.DefaultCodeFormatterConstants;
 
 
 /**
@@ -129,6 +139,18 @@ public class Generator
      */
     public boolean commentFormatting;
 
+    /**
+     * Whether Eclipse JDT unused import checking should be used to remove unused imports from generated Java code.
+     * @since 2.9
+     */
+    public boolean importOrganizing;
+
+    /**
+     * Whether registered cleanup actions should be applied to the generated source code.
+     * @since 2.10
+     */
+    public boolean cleanup;
+
     /**
      * Code formatter options to be used instead of the defaults for Java code formatting.
      */
@@ -181,6 +203,11 @@ public class Generator
 
   protected JControlModel jControlModel;
 
+  /**
+   * @since 2.10
+   */
+  protected Set<URI> generatedOutputs = new LinkedHashSet<URI>();
+
   /**
    * Creates a generator that delegates directly to the {@link GeneratorAdapterFactory.Descriptor.Registry#INSTANCE global}
    * adapter factory descriptor registry.
@@ -303,7 +330,8 @@ public class Generator
       {
         if (input instanceof GenModel)
         {
-          switch (((GenModel)input).getComplianceLevel())
+          GenModel genModel = (GenModel)input;
+          switch (genModel.getComplianceLevel())
           {
             case JDK14_LITERAL:
             {
@@ -325,6 +353,58 @@ public class Generator
               facadeHelper.setCompilerCompliance(JavaCore.VERSION_1_7); 
               break;
             }
+            case JDK80_LITERAL:
+            {
+              facadeHelper.setCompilerCompliance("1.8"); 
+              break;
+            }
+            case JDK90_LITERAL:
+            {
+              facadeHelper.setCompilerCompliance("9"); 
+              break;
+            }
+            case JDK100_LITERAL:
+            {
+              facadeHelper.setCompilerCompliance("10"); 
+              break;
+            }
+            case JDK110_LITERAL:
+            {
+              facadeHelper.setCompilerCompliance("11"); 
+              break;
+            }
+            case JDK120_LITERAL:
+            {
+              facadeHelper.setCompilerCompliance("12"); 
+              break;
+            }
+          }
+
+          if (genModel.isCodeFormatting())
+          {
+            jControlModel.setLeadingTabReplacement(null);
+            jControlModel.setConvertToStandardBraceStyle(false);
+          }
+          else
+          {
+            Map<String, String> options = GenModelUtil.getJavaOptions(genModel);
+            String tabSize = options.get(DefaultCodeFormatterConstants.FORMATTER_TAB_SIZE);
+            String braceStyle = options.get(DefaultCodeFormatterConstants.FORMATTER_BRACE_POSITION_FOR_TYPE_DECLARATION);
+            String tabCharacter = options.get(DefaultCodeFormatterConstants.FORMATTER_TAB_CHAR);
+            if (JavaCore.TAB.equals(tabCharacter))
+            {
+              jControlModel.setLeadingTabReplacement("\t");
+            }
+            else
+            {
+              String spaces = "";
+              for (int i = Integer.parseInt(tabSize); i > 0; --i)
+              {
+                spaces += " ";
+              }
+              jControlModel.setLeadingTabReplacement(spaces);
+            }
+            jControlModel.setConvertToStandardBraceStyle(DefaultCodeFormatterConstants.END_OF_LINE.equals(braceStyle));
           }
         }
       }
@@ -658,6 +738,15 @@ public class Generator
       {
         result.add(data[i].adapter.postGenerate(data[i].object, projectType));
       }
+
+      // Optionally invoke any source cleanup actions.
+      // This is only possible if JDT and JDT UI are available.
+      //
+      if (getOptions().cleanup && CommonPlugin.IS_RESOURCES_BUNDLE_AVAILABLE && !generatedOutputs.isEmpty() && jControlModel != null && jControlModel.getFacadeHelper() != null)
+      {
+        EclipseHelper.sourceCleanup(generatedOutputs);
+      }
+
       return result;
     }
     finally
@@ -667,6 +756,62 @@ public class Generator
     }
   }
 
+  /**
+   * Clients are not expect to implement this interface.
+   * It can only be implemented if the JDT UI is available, because source cleanup actions are implemented there.
+   * @since 2.10
+   */
+  public interface CleanupScheduler
+  {
+    void schedule(Set<ICompilationUnit> compilationUnits);
+  }
+
+  private static class EclipseHelper
+  {
+    private static final CleanupScheduler SCHEDULER;
+    static
+    {
+      CleanupScheduler cleanupScheduler = null;
+      try
+      {
+        Class<?> generatorUIUtilClass = CommonPlugin.loadClass("org.eclipse.emf.codegen.ecore.ui", "org.eclipse.emf.codegen.ecore.genmodel.presentation.GeneratorUIUtil");
+        cleanupScheduler = (CleanupScheduler)generatorUIUtilClass.getField("CLEANUP_SCHEDULER").get(null);
+      }
+      catch (Exception exception)
+      {
+        // Ignore
+      }
+
+      SCHEDULER = cleanupScheduler;
+    }
+
+    public static void sourceCleanup(final Set<URI> generatedOutputs)
+    {
+      if (SCHEDULER != null)
+      {
+        IWorkspaceRoot workspaceRoot = EcorePlugin.getWorkspaceRoot();
+        if (workspaceRoot != null)
+        {
+          Set<ICompilationUnit> compilationUnits = new LinkedHashSet<ICompilationUnit>();
+          for (URI generatedOutput : generatedOutputs)
+          {
+            if ("java".equals(generatedOutput.fileExtension()))
+            {
+              IFile file = workspaceRoot.getFile(new Path(generatedOutput.toString()));
+              ICompilationUnit compilationUnit = JavaCore.createCompilationUnitFrom(file);
+              if (compilationUnit.getJavaProject().isOnClasspath(compilationUnit))
+              {
+                compilationUnits.add(compilationUnit);
+              }
+            }
+          }
+
+          SCHEDULER.schedule(compilationUnits);
+        }
+      }
+    }
+  }
+
   /**
    * Determines whether code generation can continue, based on the given <code>Diagnostic</code>. This implementation
    * returns true unless the {@link org.eclipse.emf.common.util.Diagnostic#getSeverity severity} is
@@ -679,6 +824,28 @@ public class Generator
     return diagnostic.getSeverity() != Diagnostic.CANCEL;
   }
 
+  /**
+   * A {@link AbstractGeneratorAdapter#createOutputStream(URI) callback} indicating that output was generated for the given workspace path.
+   * @see #getGeneratedOutputs()
+   * @see AbstractGeneratorAdapter#createOutputStream(URI)
+   * @since 2.10
+   */
+  public void generatedOutput(URI workspacePath)
+  {
+    generatedOutputs.add(workspacePath);
+  }
+
+  /**
+   * Returns the workspace paths for which outputs have been {@link AbstractGeneratorAdapter#createOutputStream(URI) generated}.
+   * @since 2.10
+   * @see #generatedOutput(URI)
+   * @see AbstractGeneratorAdapter#createOutputStream(URI)
+   */
+  public Set<URI> getGeneratedOutputs()
+  {
+    return generatedOutputs;
+  }
+
   /**
    * Disposes all of the generator's adapter factories, by calling {@link GeneratorAdapterFactory#dispose() dispose()}
    * on each.
